@@ -23,13 +23,12 @@
 #include <iostream>
 #include <iomanip>
 
-// Global objects are usually a bad thing. For the purpose of this example, we will use them, however.
 Glib::RefPtr<Glib::MainLoop> mainloop;
 Glib::RefPtr<Gst::Pipeline> pipeline;
 Glib::RefPtr<Gst::Element> decoder;
-gulong data_probe_id;
+gulong data_probe_id = 0;
 
-bool print_stream_position(void)
+bool on_timeout()
 {
   Gst::Format fmt = Gst::FORMAT_TIME;
   gint64 pos = 0;
@@ -38,12 +37,14 @@ bool print_stream_position(void)
   Glib::RefPtr<Gst::Query> query = Gst::QueryPosition::create(fmt);
 
   if (pipeline->query(query)
-    && pipeline->query_duration(fmt, len)) {
-
-    Glib::RefPtr<Gst::QueryPosition> posQuery =
+    && pipeline->query_duration(fmt, len))
+  {
+    // Cast query's RefPtr to RefPtr<Gst::QueryPosition> to parse the
+    // pipeline's position query with the Gst::QueryPosition::parse() method
+    Glib::RefPtr<Gst::QueryPosition> query_pos =
       Glib::RefPtr<Gst::QueryPosition>::cast_dynamic(query);
-
-    posQuery->parse(fmt, pos);
+    if(query_pos)
+      query_pos->parse(fmt, pos);
 
     std::cout << std::right << "Time: " << std::setfill('0') <<
       std::setw(3) << Gst::get_hours(pos) << ":" <<
@@ -63,7 +64,7 @@ bool print_stream_position(void)
 }
 
 // This function is used to receive asynchronous messages in the main loop.
-bool on_bus_message(const Glib::RefPtr<Gst::Bus>& /* bus_not_used */, const Glib::RefPtr<Gst::Message>& message)
+bool on_bus_message(const Glib::RefPtr<Gst::Bus>& /* bus */, const Glib::RefPtr<Gst::Message>& message)
 {
   switch (message->get_message_type()) {
     case Gst::MESSAGE_EOS:
@@ -101,7 +102,15 @@ void on_parser_pad_added(const Glib::RefPtr<Gst::Pad>& newPad)
   // We can now link this pad with the audio decoder
   std::cout << "Dynamic pad created. Linking parser/decoder." << std::endl;
   Glib::RefPtr<Gst::Pad> sinkPad = decoder->get_pad("sink");
-  newPad->link(sinkPad);
+
+  try
+  {
+    newPad->link(sinkPad);
+  }
+  catch(const std::runtime_error& ex)
+  {
+    std::cerr << "Exception caught while linking: " << ex.what() << std::endl;
+  }
 }
 
 bool on_sink_pad_have_data(const Glib::RefPtr<Gst::Pad>& pad,
@@ -115,65 +124,74 @@ bool on_sink_pad_have_data(const Glib::RefPtr<Gst::Pad>& pad,
 
 int main(int argc, char* argv[])
 {
-  // check input arguments
+  // Check input arguments:
   if (argc < 2)
   {
     std::cout << "Usage: " << argv[0] << " <Ogg/Vorbis filename>" << std::endl;
     return -1;
   }
 
-  // initialize Gstreamermm
+  const std::string filename = argv[1];
+
+  // Initialize Gstreamermm:
   Gst::init(argc, argv);
 
   mainloop = Glib::MainLoop::create();
 
-  // Create the pipeline
+  // Create the pipeline:
   pipeline = Gst::Pipeline::create("audio-player");
-  std::cout << "pipeline=" << pipeline << std::endl;
 
-  // Create the elements
-  // Reads file from disk
-  Glib::RefPtr<Gst::Element> source = Gst::ElementFactory::create("filesrc", "file-source");
-  std::cout << "source=" << source << std::endl;
+  // Create the elements:
 
-  // Parses the ogg streams into elementary streams (note that an ogg file may contain a video stream too)
-  Glib::RefPtr<Gst::Element> parser = Gst::ElementFactory::create("oggdemux", "ogg-parser");
-  std::cout << "parser=" << parser << std::endl;
+  // filsrc reads the file from disk:
+  Glib::RefPtr<Gst::Element> source = Gst::ElementFactory::create_element("filesrc");
+  if(!source)
+    std::cerr << "filesrc element could not be created." << std::endl;
 
-  // Decodes a vorbis stream
-  decoder = Gst::ElementFactory::create("vorbisdec", "vorbis-decoder");
-  std::cout << "decoder=" << decoder << std::endl;
+  // oggdemux parses the ogg streams into elementary streams (audio and video):
+  Glib::RefPtr<Gst::Element> parser = Gst::ElementFactory::create_element("oggdemux");
+  if(!parser)
+    std::cerr << "oggdemux element could not be created." << std::endl;
 
-  // Converts audio() to a format which can be used by the next element
-  Glib::RefPtr<Gst::Element> conv = Gst::ElementFactory::create("audioconvert", "converter");
-  std::cout << "conv=" << conv << std::endl;
+  // vorbisdec decodes a vorbis (audio) stream:
+  decoder = Gst::ElementFactory::create_element("vorbisdec");
+  if(!decoder)
+    std::cerr << "vorbisdec element could not be created." << std::endl;
+
+  // audioconvert converts raw audio to a format which can be used by the next element
+  Glib::RefPtr<Gst::Element> conv = Gst::ElementFactory::create_element("audioconvert");
+  if(!conv)
+    std::cerr << "audioconvert element could not be created." << std::endl;
 
   // Outputs sound to an ALSA audio device
-  Glib::RefPtr<Gst::Element> sink = Gst::ElementFactory::create("alsasink", "alsa-output");
-  std::cout << "sink=" << sink << std::endl;
+  Glib::RefPtr<Gst::Element> sink = Gst::ElementFactory::create_element("alsasink");
+  if(!sink)
+    std::cerr << "alsasink element could not be created." << std::endl;
 
+  //Check that the elements were created:
   if (!pipeline || !source || !parser || !decoder || !conv || !sink)
   {
     std::cerr << "One element could not be created" << std::endl;
     return -1;
   }
 
-  data_probe_id = sink->get_pad("sink")->add_data_probe(
-    sigc::ptr_fun(&on_sink_pad_have_data));
-  std::cout << "sink data probe id = " << data_probe_id << std::endl;
+  Glib::RefPtr<Gst::Pad> pad = sink->get_pad("sink");
+  if(pad)
+    data_probe_id = pad->add_data_probe( sigc::ptr_fun(&on_sink_pad_have_data) );
+  //std::cout << "sink data probe id = " << data_probe_id << std::endl;
 
 
-  // //eireuo
-  // Set filename property on the file source. Also add a message handler:
-  source->set_property("location", std::string(argv[1]));
+  // Set the filename property on the file source.
+  // TODO: Create a FileSrc class that we can dynamic_cast<> to, so we can use property_location()?
+  source->set_property("location", filename);
 
-  // get the bus from the pipeline
+  // Get the bus from the pipeline, 
+  // and add a bus watch to the default main context with the default priority:
   Glib::RefPtr<Gst::Bus> bus = pipeline->get_bus();
-
-  // Add a bus watch to the default main context with the default priority.
   bus->add_watch( sigc::ptr_fun(&on_bus_message) );
 
-  // Put all elements in a pipeline:
+
+  // Put all the elements in a pipeline:
   try
   {
     pipeline->add(source)->add(parser)->add(decoder)->add(conv)->add(sink);
@@ -184,21 +202,30 @@ int main(int argc, char* argv[])
     return -1;
   }
 
-  // Link together:
-  source->link(parser);
 
-  // We cannot link the parser and decoder yet, 
-  // because the parser uses dynamic pads. For that,
-  // we set a pad-added signal handler:
-  parser->signal_pad_added().connect( sigc::ptr_fun(&on_parser_pad_added) );
+  // Link the elements together:
+  try
+  {
+    source->link(parser);
 
-  decoder->link(conv)->link(sink);
+    // We cannot link the parser and decoder yet, 
+    // because the parser uses dynamic pads.
+    // So we do it later in a pad-added signal handler:
+    parser->signal_pad_added().connect( sigc::ptr_fun(&on_parser_pad_added) );
 
-  // Call print_stream_position function at a 200ms
+    decoder->link(conv)->link(sink);
+  }
+  catch(const std::runtime_error& ex)
+  {
+    std::cout << "Exception while linking elements: " << ex.what() << std::endl;
+  }
+
+
+  // Call on_timeout function at a 200ms
   // interval to regularly print the position of the stream
-  Glib::signal_timeout().connect(sigc::ptr_fun(&print_stream_position), 200);
+  Glib::signal_timeout().connect(sigc::ptr_fun(&on_timeout), 200);
 
-  // Now set to playing and iterate:
+  // Now set the whole pipeline to playing and start the main loop:
   std::cout << "Setting to PLAYING." << std::endl;
   pipeline->set_state(Gst::STATE_PLAYING);
   std::cout << "Running." << std::endl;
