@@ -25,11 +25,9 @@
 #include "event.h"
 #include "filter.h"
 #include "format.h"
-#include "ghostpad.h"
 #include "index.h"
 #include "indexfactory.h"
 #include "interface.h"
-#include "iterator.h"
 #include "message.h"
 #include "pad.h"
 #include "padtemplate.h"
@@ -39,6 +37,7 @@
 #include "pluginfeature.h"
 #include "query.h"
 #include "registry.h"
+#include "segment.h"
 #include "structure.h"
 #include "systemclock.h"
 #include "tagsetter.h"
@@ -46,6 +45,10 @@
 #include "urihandler.h"
 #include "value.h"
 #include "xml.h"
+#include "basesrc.h"
+#include "basesink.h"
+#include "basetransform.h"
+#include "pushsrc.h"
 
 extern "C"
 {
