@@ -26,9 +26,11 @@ int main (int argc, char* argv[])
 {
   Gst::init(argc, argv);
 
-  Glib::RefPtr<Gst::Element> element = Gst::ElementFactory::create("fakesrc", "source");
+  Glib::RefPtr<Gst::Element> element = Gst::ElementFactory::create_element("fakesrc", "source");
 
   if(element)
     std::cout << "Successfully created gst element '" <<
       element->get_name() << "'." << std::endl;
+
+  return 0;
 }
